"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require("reflect-metadata");
var inversify_config_1 = require("../inversify.config");
var chai_1 = require("chai");
var sinon = require("sinon");
var _ = require("lodash");
var path = require("path");
var pathToScripts = path.resolve(__dirname, '../../ts/test/shell-scripts');
var testScript = path.resolve(pathToScripts, 'kitchen-sink.sh');
var testArgs = [
    'arg1',
    'arg2'
];
function checkError(error, code, signal, stdOut, stdErr) {
    if (code === void 0) { code = 0; }
    if (signal === void 0) { signal = ''; }
    if (stdOut === void 0) { stdOut = ''; }
    if (stdErr === void 0) { stdErr = ''; }
    chai_1.expect(error).to.exist;
    chai_1.expect(error.message).to.be.not.empty;
    checkResult(error.message, code, signal, stdOut, stdErr);
}
function checkResult(result, code, signal, stdOut, stdErr) {
    if (code === void 0) { code = 0; }
    if (signal === void 0) { signal = ''; }
    if (stdOut === void 0) { stdOut = ''; }
    if (stdErr === void 0) { stdErr = ''; }
    var resultObj = JSON.parse(result);
    chai_1.expect(resultObj.code).to.equal(code);
    chai_1.expect(resultObj.signal).to.equal(signal);
    chai_1.expect(resultObj.stderrText).to.equal(stdErr);
    chai_1.expect(resultObj.stdoutText).to.equal(stdOut);
}
function getNewSpawnOptions() {
    return _.clone({
        preSpawnMessage: 'PreSpawn Message',
        postSpawnMessage: 'PostSpawn Message',
        suppressDiagnostics: true,
        suppressStdErr: true,
        suppressStdOut: true,
        cacheStdErr: true,
        cacheStdOut: true,
        suppressResult: true,
        suppressFinalError: true
    });
}
function getArgArray() {
    return [
        testScript,
        'writeToStdOutErrExitWithErrCode',
        '0',
        'writeToStdOut',
        'writeToStdErr'
    ].concat(testArgs);
}
describe('Testing Spawn Creation/Force Error', function () {
    it('should be created by kernel', function (done) {
        var spawn = inversify_config_1.default.get('Spawn');
        chai_1.expect(spawn).to.exist;
        done();
    });
    it('should have final callback with error', function (done) {
        var spawn = inversify_config_1.default.get('Spawn');
        spawn.forceError = true;
        spawn.spawnShellCommandAsync(null, null, null, function (err) {
            chai_1.expect(err).to.exist;
            chai_1.expect(err.message).to.equal('force error: spawnShellCommandAsync');
            done();
        });
    });
});
describe('Testing Spawn ', function () {
    var spawn;
    var argArray;
    var spawnOptions;
    var sinonSandbox;
    var cbStdOutSpy;
    var cbStdErrSpy;
    var cbDiagnosticSpy;
    function cbStatusMock(err, result) {
        if (err) {
            cbStdErrSpy(err, result);
        }
        else {
            cbStdOutSpy(err, result);
        }
    }
    var o = {
        cbStdOutCall: function () {
        },
        cbStdErrCall: function () {
        },
        cbStatus: function () {
        },
        cbDiagnostic: function () {
        }
    };
    beforeEach(function () {
        spawn = inversify_config_1.default.get('Spawn');
        argArray = getArgArray();
        spawnOptions = getNewSpawnOptions();
        sinonSandbox = sinon.sandbox.create();
        cbDiagnosticSpy = sinonSandbox.spy(o, 'cbDiagnostic');
        cbStdOutSpy = sinonSandbox.spy(o, 'cbStdOutCall');
        cbStdErrSpy = sinonSandbox.spy(o, 'cbStdErrCall');
    });
    afterEach(function () {
        sinonSandbox.restore();
    });
    it('no result, exit with code 0', function (done) {
        argArray[1] = 'exitWithErrCode';
        spawn.spawnShellCommandAsync(argArray, spawnOptions, o.cbStatus, function (err, result) {
            chai_1.expect(err).to.not.exist;
            chai_1.expect(result).to.be.empty;
            chai_1.expect(cbStdErrSpy.called).to.be.false;
            chai_1.expect(cbStdOutSpy.called).to.be.false;
            chai_1.expect(cbDiagnosticSpy.called).to.be.false;
            done();
        }, o.cbDiagnostic);
    });
    it('no result, yes diagnostics, exit with code 0', function (done) {
        argArray[1] = 'exitWithErrCode';
        spawnOptions.suppressDiagnostics = false;
        spawn.spawnShellCommandAsync(argArray, spawnOptions, o.cbStatus, function (err, result) {
            chai_1.expect(err).to.not.exist;
            chai_1.expect(result).to.be.empty;
            chai_1.expect(cbStdErrSpy.called).to.be.false;
            chai_1.expect(cbStdOutSpy.called).to.be.false;
            chai_1.expect(cbDiagnosticSpy.called).to.be.true;
            done();
        }, o.cbDiagnostic);
    });
    it('yes result, exit with code 0', function (done) {
        argArray[1] = 'exitWithErrCode';
        spawnOptions.suppressResult = false;
        spawn.spawnShellCommandAsync(argArray, spawnOptions, o.cbStatus, function (err, result) {
            chai_1.expect(err).to.not.exist;
            checkResult(result);
            chai_1.expect(cbStdErrSpy.called).to.be.false;
            chai_1.expect(cbStdOutSpy.called).to.be.false;
            chai_1.expect(cbDiagnosticSpy.called).to.be.false;
            done();
        }, o.cbDiagnostic);
    });
    it('no result, no finalError, exit with code 3', function (done) {
        argArray[1] = 'exitWithErrCode';
        argArray[2] = '3';
        spawn.spawnShellCommandAsync(argArray, spawnOptions, o.cbStatus, function (err, result) {
            chai_1.expect(err).to.not.exist;
            chai_1.expect(result).to.be.empty;
            chai_1.expect(cbStdErrSpy.called).to.be.false;
            chai_1.expect(cbStdOutSpy.called).to.be.false;
            chai_1.expect(cbDiagnosticSpy.called).to.be.false;
            done();
        }, o.cbDiagnostic);
    });
    it('no result, yes finalError, exit with code 3', function (done) {
        argArray[1] = 'exitWithErrCode';
        argArray[2] = '3';
        spawnOptions.suppressFinalError = false;
        spawn.spawnShellCommandAsync(argArray, spawnOptions, o.cbStatus, function (err, result) {
            checkError(err, 3);
            chai_1.expect(result).to.be.empty;
            chai_1.expect(cbStdErrSpy.called).to.be.false;
            chai_1.expect(cbStdOutSpy.called).to.be.false;
            chai_1.expect(cbDiagnosticSpy.called).to.be.false;
            done();
        }, o.cbDiagnostic);
    });
    it('yes result, yes finalError, exit with code 3', function (done) {
        argArray[1] = 'exitWithErrCode';
        argArray[2] = '3';
        spawnOptions.suppressFinalError = false;
        spawnOptions.suppressResult = false;
        spawn.spawnShellCommandAsync(argArray, spawnOptions, o.cbStatus, function (err, result) {
            checkError(err, 3);
            checkResult(result, 3);
            chai_1.expect(cbStdErrSpy.called).to.be.false;
            chai_1.expect(cbStdOutSpy.called).to.be.false;
            chai_1.expect(cbDiagnosticSpy.called).to.be.false;
            done();
        }, o.cbDiagnostic);
    });
    it('yes result, no finalError, exit with code 3', function (done) {
        argArray[1] = 'exitWithErrCode';
        argArray[2] = '3';
        spawnOptions.suppressResult = false;
        spawn.spawnShellCommandAsync(argArray, spawnOptions, o.cbStatus, function (err, result) {
            chai_1.expect(err).to.not.exist;
            checkResult(result, 3);
            chai_1.expect(cbStdErrSpy.called).to.be.false;
            chai_1.expect(cbStdOutSpy.called).to.be.false;
            chai_1.expect(cbDiagnosticSpy.called).to.be.false;
            done();
        }, o.cbDiagnostic);
    });
    it('yes result, no finalError, yes stdout, yes stderr, exit with code 0', function (done) {
        spawnOptions.suppressStdOut = false;
        spawnOptions.suppressStdErr = false;
        spawnOptions.suppressResult = false;
        spawn.spawnShellCommandAsync(argArray, spawnOptions, cbStatusMock, function (err, result) {
            chai_1.expect(err).to.not.exist;
            var concatArgs = testArgs.join('');
            checkResult(result, 0, '', concatArgs, concatArgs);
            chai_1.expect(cbStdErrSpy.called).to.be.true;
            chai_1.expect(cbStdOutSpy.called).to.be.true;
            chai_1.expect(cbDiagnosticSpy.called).to.be.false;
            done();
        }, o.cbDiagnostic);
    });
    it('edge: yes suppressStdErr, no cacheStdErr', function (done) {
        spawnOptions.suppressResult = false;
        spawnOptions.cacheStdOut = false;
        spawnOptions.cacheStdErr = false;
        spawn.spawnShellCommandAsync(argArray, spawnOptions, cbStatusMock, function (err, result) {
            chai_1.expect(err).to.not.exist;
            checkResult(result);
            chai_1.expect(cbStdErrSpy.called).to.be.false;
            chai_1.expect(cbStdOutSpy.called).to.be.false;
            chai_1.expect(cbDiagnosticSpy.called).to.be.false;
            done();
        }, o.cbDiagnostic);
    });
    it("edge: fire 'error' event on child process", function (done) {
        spawnOptions.suppressResult = false;
        spawnOptions.suppressFinalError = false;
        var cp = spawn.spawnShellCommandAsync(argArray, spawnOptions, o.cbStatus, function (err, result) {
            checkResult(result, 30, 'SIGUSR1');
            checkError(err, 30, 'SIGUSR1');
            chai_1.expect(cbStdErrSpy.called).to.be.false;
            chai_1.expect(cbStdOutSpy.called).to.be.false;
            chai_1.expect(cbDiagnosticSpy.called).to.be.false;
            done();
        }, o.cbDiagnostic);
        cp.emit('error', 30, 'SIGUSR1');
    });
    it("edge: fire 'error' event on child process", function (done) {
        argArray[0] = '/';
        spawnOptions.suppressResult = false;
        spawnOptions.suppressFinalError = false;
        spawn.spawnShellCommandAsync(argArray, spawnOptions, o.cbStatus, function (err, result) {
            chai_1.expect(result).to.not.exist;
            chai_1.expect(err['code']).to.equal('EACCES');
            chai_1.expect(err['errno']).to.equal('EACCES');
            chai_1.expect(err.message).to.equal('spawn EACCES');
            chai_1.expect(cbStdErrSpy.called).to.be.false;
            chai_1.expect(cbStdOutSpy.called).to.be.false;
            chai_1.expect(cbDiagnosticSpy.called).to.be.false;
            done();
        }, o.cbDiagnostic);
    });
});
//# sourceMappingURL=spawn.test.js.map